#!/usr/bin/env python
# USP Applications plugin

try:
    import pygtk
    pygtk.require( "2.0" )
except:
    pass
import gtk
import gtk.glade
import sys
import os
import gobject
import gconf
import gnomevfs
import string
import fnmatch
import gmenu
import pango
import re

from easyfiles import *
from easybuttons import *
from execute import *
from easygconf import *
from usp_util import *

class Menu:
    def __init__( self, MenuToLookup ):
        self.tree = gmenu.lookup_tree( MenuToLookup )
        self.directory = self.tree.get_root_directory()

    def getMenus( self, parent=None ):
        if parent == None:
            #gives top-level "Applications" item
            yield self.tree.root
        else:
            for menu in parent.get_contents():
                if menu.get_type() == gmenu.TYPE_DIRECTORY and self.__isVisible( menu ):
                    yield menu

    def getItems( self, menu ):
        for item in menu.get_contents():
            if item.get_type() == gmenu.TYPE_ENTRY and item.get_desktop_file_id()[-19:] != '-usercustom.desktop' and self.__isVisible( item ):
                yield item

    def __isVisible( self, item ):
        if item.get_type() == gmenu.TYPE_ENTRY:
            return not ( item.get_is_excluded() or item.get_is_nodisplay() )
        if item.get_type() == gmenu.TYPE_DIRECTORY and len( item.get_contents() ):
            return True


class EditDialog:
    def __init__( self ):
        # Set the Glade file
        self.gladefile = os.path.join( os.path.dirname( __file__ ), "applications.glade" )

    def run( self, Data ):
        """This function will show the Edit Dialog"""
        self.wTree = gtk.glade.XML( self.gladefile, "edit_dialog" )
        #Get the actual dialog widget
        self.dlg = self.wTree.get_widget( "edit_dialog" )

        self.wTree.get_widget( "edit_name_entry" ).set_text( Data[2][:-1][Data[2][:-1].find( "=" )+1:] )
        self.wTree.get_widget( "edit_genericname_entry" ).set_text( Data[6][:-1][Data[6][:-1].find( "=" )+1:] )
        self.wTree.get_widget( "edit_comment_entry" ).set_text( Data[3][:-1][Data[3][:-1].find( "=" )+1:] )
        self.wTree.get_widget( "edit_exec_entry" ).set_text( Data[4][:-1][Data[4][:-1].find( "=" )+1:] )
        self.wTree.get_widget( "edit_icon_entry" ).set_text( Data[5][:-1][Data[5][:-1].find( "=" )+1:] )
        #print Data[7][:-1][Data[7][:-1].find( "=" )+1:]
        if Data[7][:-1][Data[7][:-1].find( "=" )+1:] == "true":
        	self.wTree.get_widget( "TermChkBtn" ).set_active( True )
        else:
        	self.wTree.get_widget( "TermChkBtn" ).set_active( False )
        self.dlg.present()

        #run the dialog and store the response
        self.result = self.dlg.run()
        if self.result == -5:
            InTerm = "false"
            if self.wTree.get_widget( "TermChkBtn" ).get_active() == True:
                InTerm = "true"
            GotData = [self.wTree.get_widget( "edit_name_entry" ).get_text(), \
            self.wTree.get_widget( "edit_comment_entry" ).get_text(), \
            self.wTree.get_widget( "edit_exec_entry" ).get_text(), \
            self.wTree.get_widget( "edit_icon_entry" ).get_text(), \
            self.wTree.get_widget( "edit_genericname_entry" ).get_text(), \
            InTerm]
        else:
            GotData = [""]

        self.dlg.destroy()
        return GotData

class pluginclass:
    TARGET_TYPE_TEXT = 80
    toButton = [ ( "text/uri-list", 0, TARGET_TYPE_TEXT ) ]

    def __init__( self, USPWin ):

	self.USPWin = USPWin
        # The Glade file for the plugin
        self.gladefile = os.path.join( os.path.dirname( __file__ ), "applications.glade" )

        # Read GLADE file
        self.wTree = gtk.glade.XML( self.gladefile, "window1" )

        # These properties are NECESSARY to maintain consistency
        # throughout USP

        # Set 'window' property for the plugin (Must be the root widget)
        self.window = self.wTree.get_widget( "window1" )

        # Set 'heading' property for plugin
        self.heading = "Applications"

        self.ApplicationTips = gtk.Tooltips()

        # This should be the first item added to the window in glade
        self.content_holder = self.wTree.get_widget( "Applications" )
        self.USPWin.wTree.get_widget('window1').connect("hide", self.RemTab)

        # Items to get custom colors
        self.itemstocolor = [self.wTree.get_widget( "viewport2" ), self.wTree.get_widget( "viewport3" )]

        # Gconf stuff
        self.gconf_dir = '/apps/usp/plugins/applications'
        self.client = gconf.client_get_default()
        self.client.add_dir( '/apps/usp/plugins/applications', gconf.CLIENT_PRELOAD_NONE )
        self.client.notify_add( '/apps/usp/plugins/applications/execute_app', self.RegenPlugin )
        self.client.notify_add( '/apps/usp/plugins/applications/do_not_filter', self.RegenPlugin )
        self.client.notify_add( '/apps/usp/plugins/applications/icon_size', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/sub_text_size', self.RegenPlugin )
        self.client.notify_add( '/apps/usp/plugins/applications/height', self.RebuildPlugin )
        self.client.notify_add( '/apps/usp/plugins/applications/width', self.RebuildPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/swap_generic_name', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/appgeneric_font_size', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/appname_font_size', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/remember_last_shown_tab', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/menu_style', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/hide_search_box', self.HideEntry )
	self.client.notify_add( '/apps/usp/plugins/applications/show_allapps_on_start', self.RegenPlugin )
	self.client.notify_add( '/apps/usp/plugins/applications/show_category_button_icon', self.RegenPlugin )

        self.wTree.get_widget( "applications_button_holder" ).connect( "drag_data_received", self.ReceiveCallback )
        self.wTree.get_widget( "applications_button_holder" ).drag_dest_set( gtk.DEST_DEFAULT_MOTION |
                             gtk.DEST_DEFAULT_HIGHLIGHT |
                             gtk.DEST_DEFAULT_DROP,
                             self.toButton, gtk.gdk.ACTION_COPY )


	self.wTree.get_widget( "entry1" ).connect("key-press-event", self.sorted)
        self.wTree.get_widget( "button7" ).connect( "button_release_event", self.Search )
        self.wTree.get_widget( "button7" ).connect( "key_press_event", self.PressSearch )

        self.GetGconfEntries()
        # Setup Monitor of Favourite applications list
        self.app_monitor = FileMonitor(os.path.expanduser("~/.usp/applications.list"))
	self.app_monitor.open()
	self.app_monitor.connect("created", self.do_applications_file)
	self.app_monitor.connect("changed", self.do_applications_file)


    def sorted(self, widget, event):
	if event.keyval == 65293:
		VisibleList = []
		for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
			if i.get_property('visible') == True:
				VisibleList.append(i)

	    	if self.USPWin.pinmenu == False:
	    		self.USPWin.wTree.get_widget( "window1" ).hide()
			
	    	self.wTree.get_widget( "entry1" ).grab_focus()
		try:
		    Execute( widget, VisibleList[0].Label4.split(), False)
		except:
		    command = self.wTree.get_widget( "entry1" ).get_text()
		    try:
		        if command[0:1] == "$":
		        	Execute( None, command[1:].split(), True )
		        else:
		        	os.system(command+ ' &')
		    except:
		        dia = gtk.Dialog('File not found!',
		                         None,  #the toplevel wgt of your app
		                         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,  #binary flags or'ed together
		                         ("Ok", gtk.RESPONSE_OK))
		        dia.vbox.pack_start(gtk.Label(command+' could not be run!'))
		        dia.vbox.show_all()
		        dia.show()
		        result = dia.run()
			if result == gtk.RESPONSE_OK:
		            dia.destroy()
		        pass
		self.wTree.get_widget( "entry1" ).set_text("")
		self.ChangeTab( "", "", 0 )
		
		
    def RegenPlugin( self, *args, **kargs ):
        self.GetGconfEntries()
        self.do_applications_file()
	self.Todos()
	self.HideEntry()

    def GetGconfEntries( self ):
        self.client = gconf.client_get_default()
        self.usecustomcolor = SetGconf( self.client, "bool", "/apps/usp/use_custom_color", True )
        self.customcolor = SetGconfColor( self.client, "/apps/usp/custom_color", '#C1C1C1' )
        self.appnamesize = SetGconf( self.client, "int", "/apps/usp/plugins/applications/appname_font_size", 11 )
        self.appgensize = SetGconf( self.client, "int", "/apps/usp/plugins/applications/appgeneric_font_size", 9 )

        self.execapp = SetGconf( self.client, "string", "/apps/usp/plugins/applications/execute_app", "nautilus" )
        self.width = SetGconf( self.client, "int", "/apps/usp/plugins/applications/width", 180 )
        self.height = SetGconf( self.client, "int", "/apps/usp/plugins/applications/height", 380 )
        self.donotfilterapps = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/do_not_filter", False )
        self.iconsize = SetGconf( self.client, "int", "/apps/usp/plugins/applications/icon_size", 2 )
	self.favouritesubsize = SetGconf( self.client, "int", "/apps/usp/plugins/applications/sub_text_size", 8 )
        self.totalrecent = SetGconf( self.client, "int", "/apps/usp/plugins/applications/totalrecent", 0 )
        self.swapgeneric = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/swap_generic_name", False )
	self.menustyle = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/menu_style", False )
	self.HideEntryBox = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/hide_search_box", False )
	self.showcaticon = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/show_category_button_icon", False )

        # Allow plugin to be minimized to the left plugin pane
        self.sticky = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/sticky", False )
        self.minimized = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/minimized", False )
        self.showmoreapps = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/show_more_apps_button", False )
        self.showappsonstart = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/show_allapps_on_start", False )
        self.openlasttab = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/remember_last_shown_tab", False )
        if self.showappsonstart == True:
            self.lasttab = 1
        else:
            self.lasttab = 0
        # Search tool
        self.searchtool = SetGconf( self.client, "string", '/apps/usp/plugins/applications/search_command', "gnome-search-tool --named=SEARCH_STRING --start" )

        # Plugin icon
        self.icon = SetGconf( self.client, "string", '/apps/usp/plugins/applications/icon', "applications-accessories" )

        # Hide vertical dotted separator
        self.hideseparator = SetGconf( self.client, "bool", "/apps/usp/plugins/applications/hide_separator", False )

    def SetHidden( self, state ):
	if state == True:
		WriteGconf( self.client, "bool", "/apps/usp/plugins/applications/minimized", True )
	else:
		WriteGconf( self.client, "bool", "/apps/usp/plugins/applications/minimized", False )

    def HideEntry(self, *args, **kargs):
	self.GetGconfEntries()
	if self.HideEntryBox == True:
		self.wTree.get_widget( "hbox9" ).hide()
	else:
		self.wTree.get_widget( "hbox9" ).show()	

    def RebuildPlugin(self, *args, **kargs):
        self.GetGconfEntries()
        self.content_holder.set_size_request(self.width, self.height )
	

    #Add from applications file
    def do_applications_file( self, *args, **kargs ):
        TestForDir( ".usp/applications" )
        TestForFiles( ".usp/applications.list" )
        TestForFiles( ".usp/recentapps.list" )


        applicationsFile = open ( os.path.join( os.path.expanduser( "~" ), ".usp/applications.list" ), "r" )
        applicationsList = applicationsFile.readlines()

        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            i.destroy()

        for i in range( len( applicationsList ) ):
                if ( applicationsList[i][-8:-1] == 'desktop' ):
                    ApplicationData = ParseDesktopFile( applicationsList[i] )
		    LabelOrder = [ApplicationData[4], ApplicationData[0]]
		    InTerm=False
		    if ApplicationData[5].upper()=="TRUE":
		    	InTerm=True
		    if self.swapgeneric == True:
			LabelOrder.reverse()
                    if len( ApplicationData ) > 1:
                        AppStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1 )]
                        GenStyle = [pango.AttrSize(self.appgensize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1 )]
                        Button1 = MakeAButton( self.width-10, -1, gtk.RELIEF_NONE, ApplicationData[3], self.iconsize, 48, -1, LabelOrder, [AppStyle,GenStyle] )
                        Button1.connect( "button_press_event", self.ButtonClicked, ApplicationData[2].split()  )
                        Button1.connect( "button_release_event", self.ButtonReleased, True, InTerm )
			self.ApplicationTips.set_tip( Button1, ApplicationData[1], tip_private=None )
                        Button1.MyButtonName = applicationsList[i][:-1]
                        self.wTree.get_widget( "applications_button_holder" ).pack_start( Button1, False, True, 0 )
                if applicationsList[i][:-1] == "separator":
                    self.do_separator( "" )
                if applicationsList[i][:-1] == "space":
                    self.do_space( "" )

        if len( applicationsList ) < 2 or self.showappsonstart == True:
            self.ChangeTab( "", "", 1 )

    #Add a space
    def do_space( self, widget, itemclicked=False, *args, **kargs ):
        Event1 = gtk.EventBox()
        self.SetPaneColors( [Event1] )
        Event1.show()
        self.itemstocolor.append( Event1 )
        self.wTree.get_widget( "applications_button_holder" ).pack_start( Event1, False, False, 10 )
        Event1.connect( "button_press_event", self.ButtonClicked, "" )
        Event1.connect( "button_release_event", self.ButtonReleased, True, False)
        Event1.MyButtonName = "space"

        if itemclicked:
            count = 0
            for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
                if itemclicked == i:
                    self.wTree.get_widget( "applications_button_holder" ).reorder_child( Event1, count )
                count = count + 1
            WriteListFile( ".usp/applications.list", "", "w" )
            for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
                WriteListFile( ".usp/applications.list", i.MyButtonName, "a" )

    #Add a separator
    def do_separator( self, widget, itemclicked=False, *args, **kargs ):
        sep1 = gtk.HSeparator()
        Event1 = gtk.EventBox()
        Event1.add( sep1 )
        self.SetPaneColors( [Event1] )
        Event1.show()
        sep1.show()
        self.itemstocolor.append( Event1 )
        self.wTree.get_widget( "applications_button_holder" ).pack_start( Event1, False, False, 10 )
        Event1.connect( "button_press_event", self.ButtonClicked, "" )
        Event1.connect( "button_release_event", self.ButtonReleased, True ,False)
        Event1.MyButtonName = "separator"

        if itemclicked:
            count = 0
            for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
                if itemclicked == i:
                    self.wTree.get_widget( "applications_button_holder" ).reorder_child( Event1, count )
                count = count + 1
            WriteListFile( ".usp/applications.list", "", "w" )
            for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
                WriteListFile( ".usp/applications.list", i.MyButtonName, "a" )


    def SetPaneColors( self, items ):
        if self.usecustomcolor:
            for item in items:
                item.modify_bg( gtk.STATE_NORMAL, gtk.gdk.color_parse( self.customcolor ) )

    def ChangeTab( self, widget, event, TabNum ):
        if TabNum == 0:
            self.wTree.get_widget( "notebook2" ).set_current_page( 0 )
            self.wTree.get_widget( "scrolledwindow3" ).hide()
            if self.openlasttab == True:
                self.lasttab=0

        if TabNum == 1:
            self.wTree.get_widget( "notebook2" ).set_current_page( 1 )
            self.wTree.get_widget( "scrolledwindow3" ).show()
            if self.openlasttab == True:
                self.lasttab=1

        self.wTree.get_widget( "entry1" ).grab_focus()

    def RemTab( self, *args, **kargs ):
        try:
            if len(self.wTree.get_widget('applications_button_holder').get_children()) == 0:
                self.lasttab = 1
            self.ChangeTab( "", "", self.lasttab)
        except:
            pass
		
    def Todos( self ):

        for i in self.wTree.get_widget( "vbuttonbox2" ).get_children():
            i.destroy()

        for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
            i.destroy()

        AppStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1 )]
        DirStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_BOLD,0,-1 )]

        if self.showcaticon == True:
        	CategoryButton = MakeAButton( 120, -1, gtk.RELIEF_NONE, "distributor-logo", 0, 0, 16, ["All"], [DirStyle] )
        else:
        	CategoryButton = MakeAButton( 100, -1, gtk.RELIEF_NONE, "", 0, 0, 0, ["All"], [DirStyle] )

	if self.menustyle == True:
		CategoryButton.connect( "enter_notify_event", self.Filter, "" )
	else:
        	CategoryButton.connect( "button_press_event", self.Filter, "" )

        self.wTree.get_widget( "vbuttonbox2" ).add( CategoryButton )

        self.wTree.get_widget( "entry1" ).connect( "changed", self.Filter, "", "" )
        self.wTree.get_widget( "button3" ).connect( "button_release_event", self.ChangeTab, 1 )
        self.wTree.get_widget( "button4" ).connect( "button_release_event", self.ChangeTab, 0 )

        if self.showmoreapps == True:
            self.wTree.get_widget( "vbox8" ).show()

        self.menuChangedTimer = None
        for mainitems in ['applications.menu', 'settings.menu']:
            mymenu = Menu( mainitems )
            mymenu.tree.add_monitor( self.menuChanged, None )

            for child in mymenu.directory.get_contents():
                if child.get_type() == gmenu.TYPE_DIRECTORY:
                    if self.showcaticon == True:
                        CategoryButton = MakeAButton( 120, -1, gtk.RELIEF_NONE, child.icon, 0, 0, 16, [child.name], [DirStyle] )
                    else:
                        CategoryButton = MakeAButton( 100, -1, gtk.RELIEF_NONE, "", 0, 0, 0, [child.name], [DirStyle] )

		    if self.menustyle == True:
			CategoryButton.connect( "enter_notify_event", self.Filter, child.name )
		    else:
	                CategoryButton.connect( "button_press_event", self.Filter, child.name )
                
		    self.wTree.get_widget( "vbuttonbox2" ).add( CategoryButton )
                    self.BuildLowerFiles(child)


    def menuChanged( self, x, y):
        # Wait a moment before updating the list, because it's
        # very likely that we recive a whole bunch of notifications.
        if self.menuChangedTimer:
            gobject.source_remove( self.menuChangedTimer )
            self.menuChangedTimer = None

        self.menuChangedTimer = gobject.timeout_add( 10, self.Todos )

    def Filter( self, widget, event, categorytext ):

        full_label = text = ""
        showAllAppsBool = False
        if categorytext != "" or self.donotfilterapps == False:
            self.ShowAllApps()
            self.wTree.get_widget( "entry1" ).grab_focus()
            showAllAppsBool = True

        for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
            if categorytext != "":
                showAllAppsBool = True
                #print i.Label3
                full_label = i.Label3
                text = categorytext
                self.wTree.get_widget( "entry1" ).set_text( '' )
            else:
                if self.donotfilterapps == False:
                    full_label = i.Label1 + i.Label2 + i.Label3 + i.Label4
                    text = self.wTree.get_widget( "entry1" ).get_text()
                if not re.search( text, full_label, re.IGNORECASE ):
                    i.hide()
                else:
                    i.show()


            if showAllAppsBool == True:
                if not re.search( text, full_label, re.IGNORECASE ):
                    i.hide()
                else:
                    i.show()
                if widget != self.wTree.get_widget( "entry1" ):
                    for i in self.wTree.get_widget( "vbuttonbox2" ).get_children():
                        i.set_relief( gtk.RELIEF_NONE )
                    widget.set_relief( gtk.RELIEF_HALF )

    def ShowAllApps( self, *args, **kargs ):
        self.ChangeTab( "", "", 1 )
        self.wTree.get_widget( "vbuttonbox2" ).show()

    #--------------------------------------------------------------------------------------------------
    def FolderClicked( self, widget, event, sentchild ):
        self.press_x = event.x
        self.press_y = event.y
        self.Folder = sentchild

    def FolderReleased( self, widget, ev, editable ):
        if ev.button == 1:
            if not hasattr( self, "press_x" ) or \
                not widget.drag_check_threshold( int( self.press_x ),
                                             int( self.press_y ),
                                             int( ev.x ),
                                             int( ev.y ) ):
                if self.Folder != "":
                    for i in self.wTree.get_widget( "vbuttonbox2" ).get_children():
                        i.destroy()

                    for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
                        i.destroy()

                    AppStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1 )]
                    DirStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_BOLD,0,-1 )]

                    CategoryButton = MakeAButton( 100, -1, gtk.RELIEF_NONE, "back", 0, 0, 16, ["Back"], [DirStyle] )
                    CategoryButton.connect( "button_press_event", self.BackupButtonClicked, self.Folder.parent )
                    CategoryButton.connect( "button_release_event", self.BackupButtonReleased, self.Folder.parent )
        	    self.wTree.get_widget( "vbuttonbox2" ).add( CategoryButton )
                    self.BuildLowerFiles(self.Folder)

    def BuildLowerFiles(self, FolderSent):
        self.Folder=FolderSent
        path="Error"
        iconfound = False
        itemslist = []
        buttonbox = "vbuttonbox3"
        PlaceCategories=PlaceName=PlaceComment=PlaceExec=PlaceIconName=GenericName=WhatWasDropped=BTip=""
        AppStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1 )]
        DirStyle = [pango.AttrSize(self.appnamesize*1000,0,-1),pango.AttrWeight(pango.WEIGHT_BOLD,0,-1 )]
        for grandchild in self.Folder.get_contents():
            if grandchild.get_type() == gmenu.TYPE_ENTRY or grandchild.get_type() == gmenu.TYPE_DIRECTORY:
                path = grandchild.get_desktop_file_path()

                openfile = open( GetFilePath( path ), 'r' )
                datalist = openfile.readlines()
                InTerm = False
                for i in range( len( datalist ) ):
                    datalist[i] = datalist[i].strip( '\r\n\x00' )
                    if datalist[i][0:9] == 'Terminal=':
                        if datalist[i][9:].lower() == 'true':
                            InTerm = True
                    if datalist[i][0:5] == "Name=":
                        PlaceName = datalist[i][5:]
                        BTip = datalist[i][5:]

                    if datalist[i][0:8] == "Comment=":
                        PlaceComment = datalist[i][8:]

                    if datalist[i][0:5] == "Exec=":
                        execstring = datalist[i][5:].replace( "%u", "" )
                        execstring = execstring.replace( "%U", "" )
                        execstring = execstring.replace( "%F", "" )
                        PlaceExec = execstring

                    if datalist[i][0:5] == "Icon=":
                        PlaceIconName = datalist[i][5:]
                        iconfound = True

                    if datalist[i][0:12] == "GenericName=":
                        GenericName = datalist[i][12:]

                if hasattr( grandchild , 'parent' ):
                    PlaceCategories = grandchild.parent.name

                if iconfound == False:
                    PlaceIconName = "gnome-logo-icon"

                if grandchild.get_type() == gmenu.TYPE_DIRECTORY:
                    PlaceName = " "+PlaceName
                    ItemButton = MakeAButton( 200, 35, gtk.RELIEF_NONE, "forward", 2, 40, -1, [PlaceName], [DirStyle] )
                else:
                    ItemButton = MakeAButton( 200, 35, gtk.RELIEF_NONE, PlaceIconName, 2, 40, -1, [PlaceName], [AppStyle] )

                ItemButton.Label1 = PlaceName
                ItemButton.Label2 = PlaceComment
                ItemButton.Label3 = PlaceCategories
                ItemButton.Label4 = PlaceExec
                ItemButton.MyButtonName = path
                if grandchild.get_type() == gmenu.TYPE_DIRECTORY:
                    ItemButton.connect( "button_press_event", self.FolderClicked, grandchild )
                    ItemButton.connect( "button_release_event", self.FolderReleased, "Favs" )
                else:
                    ItemButton.connect( "button_press_event", self.ButtonClicked, PlaceExec.split() )
                    ItemButton.connect( "button_release_event", self.ButtonReleased, "Favs", InTerm )

                self.ApplicationTips.set_tip( ItemButton, PlaceName + '\n' + PlaceComment, tip_private=None )

                iconfound = False
                itemslist.append( ( string.upper( PlaceName ), ItemButton ) )

        itemslist.sort()
        for i in itemslist:
            self.wTree.get_widget( buttonbox ).pack_start( i[1], False, False, 0 )

    def BackupButtonClicked( self, widget, event, parent ):
        self.press_x = event.x
        self.press_y = event.y
	self.Folder = parent

    def BackupButtonReleased( self, widget, event, parent ):
        # Checks for top level
        if hasattr(parent,"parent"):
            # Have to check again for both.
            if parent.parent.name in ["Applications","System"]:
                self.Todos()
            else:
                self.press_x = event.x
                self.press_y = event.y
                self.FolderReleased( widget, event, self.Folder )
        else:
            self.Todos()

    #--------------------------------------------------------------------------------------------------------------------

    def ButtonClicked( self, widget, event, Exec ):
        self.press_x = event.x
        self.press_y = event.y
        self.Exec = Exec

    def ButtonReleased( self, widget, ev, editable, InTerm ):
        if ev.button == 1:
            if not hasattr( self, "press_x" ) or \
                not widget.drag_check_threshold( int( self.press_x ),
                                             int( self.press_y ),
                                             int( ev.x ),
                                             int( ev.y ) ):
                if self.Exec != "" and self.Exec != None:
                    print "self.Exec = ",self.Exec
                    WriteRecentListFile( ".usp/recentapps.list", widget.MyButtonName, "a" )
		    if self.USPWin.pinmenu == False:
		    	self.USPWin.wTree.get_widget( "window1" ).hide()
		    self.wTree.get_widget( "entry1" ).grab_focus()
                    Execute( widget, self.Exec, InTerm )

        elif ev.button == 3 and editable == True:
            itemclicked = widget
            self.mTree = gtk.glade.XML( self.gladefile, "menu1" )
            x = int( ev.x )
            y = int( ev.y )
            time = ev.time
            self.mTree.get_widget( "edit1" ).connect( "activate", self.EditButton, widget, ev, self.mTree )
            self.mTree.get_widget( "remove1" ).connect( "activate", self.DelButton, widget, itemclicked, ev, self.mTree )
            self.mTree.get_widget( "move_up1" ).connect( "activate", self.MoveUp, widget, itemclicked, ev, self.mTree )
            self.mTree.get_widget( "move_down1" ).connect( "activate", self.MoveDown, widget, itemclicked, ev, self.mTree )
            self.mTree.get_widget( "insert_space1" ).connect( "activate", self.do_space, widget, itemclicked, ev, self.mTree )
            self.mTree.get_widget( "insert_separator1" ).connect( "activate", self.do_separator, widget, itemclicked, ev, self.mTree )
            self.mTree.get_widget( "menu1" ).popup( None, None, None, ev.button, time )

        elif ev.button == 3 and editable == "Favs":
            itemclicked = widget
            self.mTree = gtk.glade.XML( self.gladefile, "menu2" )
            x = int( ev.x )
            y = int( ev.y )
            time = ev.time
            self.mTree.get_widget( "menuitem1" ).connect( "activate", self.AddFavourite, widget, self.mTree )
            self.mTree.get_widget( "menu2" ).popup( None, None, None, ev.button, time )

    def DelButton( self, a, widget, itemclicked, event, menutogo, *args ):
        if itemclicked.MyButtonName not in ["space", "separator"]:
            os.spawnvp( os.P_NOWAIT, "rm", ["rm", itemclicked.MyButtonName] )
        itemclicked.destroy()
        menutogo.get_widget( "menu1" ).destroy()

        WriteListFile( ".usp/applications.list", "", "w" )
        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            WriteListFile( ".usp/applications.list", i.MyButtonName, "a" )

        self.do_applications_file()


    def EditButton( self, a, widget, event, menutogo ):
        if widget.MyButtonName not in ["space", "separator"]:
            self.USPWin.wTree.get_widget( "window1" ).hide()
            EditDlg = EditDialog()
            openfile = open( widget.MyButtonName, 'r' )
            datalist = openfile.readlines()
            result = EditDlg.run( datalist )

            if result != [""]:
                EditDesktopFile( widget.MyButtonName, result, ".usp/applications.list" )

            self.do_applications_file()
            menutogo.get_widget( "menu1" ).destroy()
            # Re-display window and depress button here

    def MoveUp( self, a, widget, itemclicked, event, menutogo, *args ):
        count = 0
        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            if itemclicked == i:
                self.wTree.get_widget( "applications_button_holder" ).reorder_child( itemclicked, count - 1 )
            count = count + 1
        menutogo.get_widget( "menu1" ).destroy()
        WriteListFile( ".usp/applications.list", "", "w" )
        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            WriteListFile( ".usp/applications.list", i.MyButtonName, "a" )

        self.do_applications_file()

    def MoveDown( self, a, widget, itemclicked, event, menutogo, *args ):
        count = 0
        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            if itemclicked == i:
                self.wTree.get_widget( "applications_button_holder" ).reorder_child( itemclicked, count + 1 )
            count = count + 1
        menutogo.get_widget( "menu1" ).destroy()
        WriteListFile( ".usp/applications.list", "", "w" )
        for i in self.wTree.get_widget( "applications_button_holder" ).get_children():
            WriteListFile( ".usp/applications.list", i.MyButtonName, "a" )

        self.do_applications_file()

    def AddToFavourites( self, location ):
        #Quick work around for not rewriting all the code :-(
        location = location + "  "


        ApplicationData = ""
        ButtonIcon = None
        # For Folders and Network Shares
        ButtonLocation = string.join( location[:-2].split( "%20" ) )
        ButtonCmd = self.execapp + ' ' + location[:-2]

        if ButtonLocation[:4] == 'file':
            ButtonIcon = "gnome-fs-directory"

        if ButtonLocation[:3] == 'smb' or ButtonLocation[:3] == 'ssh' or ButtonLocation[:7] == 'network':
            ButtonIcon = "gnome-fs-network"

        if ButtonIcon != None:
            locationName = [ButtonLocation.split( '/' )[-1:][0]]
            if locationName == [""]:
                locationName = "/"
            ApplicationData = [locationName[0], '', ButtonCmd, ButtonIcon, '', 'False']

        # For .desktop files
        if location[-9:-2] == "desktop":
            ApplicationData = ParseDesktopFile( location[:-2] )

        #For Special locations
        if location[:-2] == 'x-nautilus-desktop:///computer':
            ApplicationData = ParseDesktopFile( "/usr/share/applications/nautilus-computer.desktop" )

        if location[:-2] == 'x-nautilus-desktop:///home':
            ApplicationData = ParseDesktopFile( "/usr/share/applications/nautilus-home.desktop" )

        if location[:-2] == 'x-nautilus-desktop:///network':
            ApplicationData = ParseDesktopFile( "/usr/share/applications/network-scheme.desktop" )

        if len( ApplicationData ) > 1:
            Button1 = MakeAButton( self.width-10, -1, gtk.RELIEF_NONE, ApplicationData[3], gtk.ICON_SIZE_BUTTON, 48, -1, [ApplicationData[0]], [""] )
            Button1.connect( "button_press_event", self.ButtonClicked, ApplicationData[2].split() )
            InTerm=False
            if ApplicationData[5].upper()=="TRUE":
                InTerm=True
            Button1.connect( "button_release_event", self.ButtonReleased, True, InTerm )
            self.wTree.get_widget( "applications_button_holder" ).pack_start( Button1, False, False, 10 )


            FileExists = False
            FileData = GetFilePath( location )
            clean1 = FileData.replace( "/", "-" )
            DroppedFile = clean1.replace( " ", "" ) #+".desktop"


            for fileName in os.listdir ( os.path.join( os.path.expanduser( "~" ), ".usp/applications" ) ):
                       if fnmatch.fnmatch ( fileName, DroppedFile ):
                           FileExists = True

            if FileExists == True:
                print "File Exists!"
                Button1.destroy()
            else:
                DroppedFile = ".usp/applications/" + DroppedFile
                DroppedFile = os.path.join( os.path.expanduser( "~" ), DroppedFile )
                print "Application added - " + DroppedFile
                Button1.MyButtonName = DroppedFile
                WriteDesktopFile( DroppedFile, ApplicationData, ".usp/applications.list" )

    def AddFavourite( self, widget, button, menutogo ):
        self.AddToFavourites( button.MyButtonName )
        menutogo.get_widget( "menu2" ).destroy()
        self.do_applications_file()

    def ReceiveCallback( self, widget, context, x, y, selection, targetType, time ):
        if targetType == self.TARGET_TYPE_TEXT:
            print selection.data[:-2]
            self.AddToFavourites( selection.data[:-2] )

    def Search( self, widget, event ):
        text = self.wTree.get_widget( "entry1" ).get_text()
        self.wTree.get_widget( "entry1" ).set_text( '' )
        self.ChangeTab( "", "", 0 )
        fullstring = self.searchtool.replace( "SEARCH_STRING", text )
        newstring = fullstring.split()
        Execute( widget, newstring )

    def PressSearch( self, widget, event ):
        if event.keyval == 65293:
	        text = self.wTree.get_widget( "entry1" ).get_text()
        	self.wTree.get_widget( "entry1" ).set_text( '' )
        	self.ChangeTab( "", "", 0 )
        	fullstring = self.searchtool.replace( "SEARCH_STRING", text )
        	newstring = fullstring.split()
        	Execute( widget, newstring )

    def do_plugin( self ):
        self.do_applications_file()
        self.Todos()
	self.HideEntry()


#---------------------------------------------------------------------------------------#
# USPconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "applications.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "Applications"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/usp/plugins/applications'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/applications', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_GenFontSizeSpin_changed" : self.genfontspin,
				"on_NameFontSizeSpin_changed" : self.namefontspin,
				"on_AppIconSizeSpin_changed" : self.appiconspin,
				"on_AppsHSpin_changed" : self.apph,
				"on_AppsWSpin_changed" : self.appw,
				"on_ShowAllAppsChkBtn_toggled" : self.showallapps,
				"on_ShowCatIconChkBtn_toggled" : self.showcaticons,
				"on_MoreAppsChkBtn_toggled" : self.moreapps,
				"on_GenChkBtn_toggled" : self.genchk,
				"on_RemLastChkBtn_toggled" : self.remchk,
				"on_FiltChkBtn_toggled" : self.filtchk,
				"on_SrchCmdEntry_changed" : self.srchcmd,
				"on_HideSrchChkBtn_toggled" : self.hidesrch,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_MenuStyleChkBtn_toggled" : self.menuaction,
				"on_AppIconEntry_changed" : self.appiconchange,
				"on_StickyChkBtn_toggled" : self.appsticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("GenFontSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/applications/appgeneric_font_size',9))
		self.wTree.get_widget("NameFontSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/applications/appname_font_size',11))
		self.wTree.get_widget("AppIconSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/applications/icon_size',2))
		self.wTree.get_widget("AppsHSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/applications/height',380))
		self.wTree.get_widget("AppsWSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/applications/width',300))
		self.wTree.get_widget("ShowAllAppsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/show_allapps_on_start',False))
		self.wTree.get_widget("ShowCatIconChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/show_category_button_icon',False))
		self.wTree.get_widget("MoreAppsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/show_more_apps_button',True))
		self.wTree.get_widget("GenChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/swap_generic_name',False))
		self.wTree.get_widget("RemLastChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/remember_last_shown_tab',False))
		self.wTree.get_widget("FiltChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/do_not_filter',False))
		self.wTree.get_widget("MenuStyleChkBtn").set_active(SetGconf( self.client, "bool", "/apps/usp/plugins/applications/menu_style", False ))
		self.wTree.get_widget("HideSepChkBtn").set_active(SetGconf(self.client, 'bool', '/apps/usp/plugins/applications/hide_separator',False))
		self.wTree.get_widget("AppIconEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/applications/icon','gnome-applications'))
		self.wTree.get_widget("SrchCmdEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/applications/search_command','gnome-search-tool --named=SEARCH_STRING --start'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/applications/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def genfontspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/applications/appgeneric_font_size',int(self.wTree.get_widget("GenFontSizeSpin").get_value()))

	def namefontspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/applications/appname_font_size',int(self.wTree.get_widget("NameFontSizeSpin").get_value()))

	def apph(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/applications/height',int(self.wTree.get_widget("AppsHSpin").get_value()))

	def appw(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/applications/width',int(self.wTree.get_widget("AppsWSpin").get_value()))

	def appiconspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/applications/icon_size',int(self.wTree.get_widget("AppIconSizeSpin").get_value()))

	def showallapps(self, widget, **kargs):
		if self.wTree.get_widget("ShowAllAppsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/show_allapps_on_start',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/show_allapps_on_start',False)

	def showcaticons(self, widget, **kargs):
		if self.wTree.get_widget("ShowCatIconChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/show_category_button_icon',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/show_category_button_icon',False)

	def moreapps(self, widget, **kargs):
		if self.wTree.get_widget("MoreAppsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/show_more_apps_button',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/show_more_apps_button',False)

	def genchk(self, widget, **kargs):
		if self.wTree.get_widget("GenChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/swap_generic_name',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/swap_generic_name',False)

	def remchk(self, widget, **kargs):
		if self.wTree.get_widget("RemLastChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/remember_last_shown_tab',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/remember_last_shown_tab',False)

	def filtchk(self, widget, **kargs):
		if self.wTree.get_widget("FiltChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/do_not_filter',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/do_not_filter',False)

	def hidesrch(self, widget, **kargs):
		if self.wTree.get_widget("HideSrchChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/hide_search_box',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/hide_search_box',False)

	def srchcmd(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/applications/search_command',self.wTree.get_widget("SrchCmdEntry").get_text())

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/hide_separator',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/hide_separator',False)

	def menuaction(self, widget, **kargs):
		if self.wTree.get_widget("MenuStyleChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/menu_style',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/menu_style',False)

	def appiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("AppIconEntry").get_text()
		self.client.set_string('/apps/usp/plugins/applications/icon',self.wTree.get_widget('AppIconEntry').get_text())

	def appsticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/applications/sticky',True)
		else:
			self.client.set_bool('/apps/usp/plugins/applications/sticky',False)
