#!/usr/bin/env python
# USP Computer/Settings Plugin Version 1.0
import gtk
import gtk.glade
import sys
import os
import gobject
import datetime
import gconf
import pango
import commands
from execute import Execute
from easygconf import *

class pluginclass:
	TARGET_TYPE_TEXT = 80
	toButton = [ ( "text/uri-list", 0, TARGET_TYPE_TEXT ) ]
	"""This is the main class for the plugin"""
	"""It MUST be named pluginclass"""

	def __init__(self, USPWin):

		self.USPWin = USPWin
		#The Glade file for the plugin
		self.gladefile = os.path.join(os.path.dirname(__file__), "computer.glade")
		#self.gladefile = "/home/dave/.usp2/plugins/computer.glade"

		#Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"window1")

		#These properties are NECESSARY to maintain consistency
		#throughout USP

		#Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("window1")

		#Set 'heading' property for plugin
		self.heading = "Settings"

		#This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget("eventbox1")

		#Specify plugin width
		self.width = 250

		#Plugin icon
		self.icon = 'gnome-settings.png'

		self.gconf_dir = '/apps/usp/plugins/computer'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/computer', gconf.CLIENT_PRELOAD_NONE)
		self.client.notify_add('/apps/usp/plugins/computer/sticky',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/icon',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/settings_font_size',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/settings_info_font_size',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_computer',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_control',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_date',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_net',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_screen',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/hide_theme',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/datetime/eu_date_format',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/network_interface',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command1',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command2',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command3',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command4',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command5',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/computer/system_command6',self.RegenPlugin)

		self.RegenPlugin()
		self.SetupEvents()
		gobject.timeout_add(1000, self.date_time)

	def RegenPlugin(self, *args, **kargs):
		self.GetGconfEntries()
		self.wTree.get_widget('button1').set_size_request( self.width-2, -1 )
		self.get_gconf_info()
		self.get_proc_info()
		self.SetSystemStyle(["label2","label5","label8","label11","label14","label17"])
		self.SetInfoStyle(["label3","label4","label6","label7","label9","label10","label12","label13","label15","label16","label18","label19"])
		self.set_ip()

	def SetupEvents(self):
		#Connect event handlers
		dic = {	"on_window1_destroy" : gtk.main_quit,
				"on_button1_clicked" : (self.RunCommand,self.syscommand1),
				"on_button2_clicked" : (self.RunCommand,self.syscommand2),
				"on_button3_clicked" : (self.RunCommand,self.syscommand3),
				"on_button4_clicked" : (self.RunCommand,self.syscommand4),
				"on_button5_clicked" : (self.RunCommand,self.syscommand5),
				"on_button6_clicked" : (self.RunCommand,self.syscommand6)}
		self.wTree.signal_autoconnect(dic)



	def GetGconfEntries(self):
		self.client = gconf.client_get_default()
		self.width = SetGconf(self.client,'int','/apps/usp/plugins/computer/width',250)
		self.sticky = SetGconf( self.client, "bool", "/apps/usp/plugins/computer/sticky", False )
		self.icon = SetGconf( self.client, "string", "/apps/usp/plugins/computer/icon", "gnome-settings.png" )
		self.eudate = SetGconf(self.client,'bool','/apps/usp/plugins/datetime/eu_date_format', False)
		self.hidecomp = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_computer', False)
		self.hidecontrol = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_control', False)
		self.hidedate = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_date', False)
		self.hidenet = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_net', False)
		self.hidescreen = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_screen', False)
		self.hidetheme = SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_theme', False)
		self.netiface = SetGconf(self.client,'string','/apps/usp/plugins/computer/network_interface','eth0')
		self.settingsfontsize = SetGconf(self.client,'int','/apps/usp/plugins/computer/settings_font_size',11)
		self.infofontsize = SetGconf(self.client,'int','/apps/usp/plugins/computer/settings_info_font_size',9)
		self.syscommand1 = SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command1','gnome-system-monitor').split()
		self.syscommand2 = SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command2','gksu network-admin').split()
		self.syscommand3 = SetGconf(self.client,'string','/apps/usp/plugins/datetime/system_command1','gksu time-admin').split()
		self.syscommand4 = SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command4','gnome-theme-manager').split()
		self.syscommand5 = SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command5','gnome-display-properties').split()
		self.syscommand6 = SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command6','gcontrol').split()
		self.minimized = SetGconf( self.client, "bool", "/apps/usp/plugins/computer/minimized", False )

		# Hide vertical dotted separator
		self.hideseparator = SetGconf( self.client, "bool", "/apps/usp/plugins/computer/hide_separator", False )

	def SetHidden( self, state ):
		if state == True:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/computer/minimized", True )
		else:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/computer/minimized", False )

	def RunCommand(self, widget,TheCommand):
		self.USPWin.wTree.get_widget( "window1" ).hide()
		Execute(widget, TheCommand)

	def SetSystemStyle(self,items):
		HeadingStyle = pango.AttrList()
		attr = pango.AttrSize(self.settingsfontsize*1000,0,-1)
		HeadingStyle.insert(attr)
		attr = pango.AttrWeight(pango.WEIGHT_BOLD,0,-1)
		HeadingStyle.insert(attr)

		for i in range(len(items)):
			if self.wTree.get_widget(items[i]).get_parent().get_property( 'visible' ) == True:
				self.wTree.get_widget(items[i]).set_attributes(HeadingStyle)

	def SetInfoStyle(self,items):
		InfoStyle = pango.AttrList()
		attr = pango.AttrSize(self.infofontsize*1000,0,-1)
		InfoStyle.insert(attr)
		attr = pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1)
		InfoStyle.insert(attr)

		for i in range(len(items)):
			if self.wTree.get_widget(items[i]).get_parent().get_property( 'visible' ) == True:
				self.wTree.get_widget(items[i]).set_attributes(InfoStyle)

	def get_gconf_info(self, *args, **kwargs):
		try:
			resolution=freq=gtktheme=icontheme=None
			resolution = self.client.get_string('/desktop/gnome/screen/default/0/resolution')
			freq = self.client.get_int('/desktop/gnome/screen/default/0/rate')
			icontheme = self.client.get_string('/desktop/gnome/interface/icon_theme')
			gtktheme = self.client.get_string('/desktop/gnome/interface/gtk_theme')

			if self.hidescreen == 0:
				if resolution != None:
					self.wTree.get_widget("label15").set_label(resolution+" px")
					self.wTree.get_widget("label16").set_label("Refresh rate: "+ str(freq)+"Hz")
				self.wTree.get_widget("button5").show()
			else:
				self.wTree.get_widget("button5").hide()

			if self.hidetheme == 0:
				if gtktheme != None:
					self.wTree.get_widget("label12").set_label("Gtk: "+gtktheme)
					self.wTree.get_widget("label12").set_ellipsize(pango.ELLIPSIZE_END)
				if icontheme != None:
					self.wTree.get_widget("label13").set_label("Icon: "+icontheme)
					self.wTree.get_widget("label13").set_ellipsize(pango.ELLIPSIZE_END)
				self.wTree.get_widget("button4").show()
			else:
				self.wTree.get_widget("button4").hide()

			if self.hidecontrol == 0:
				self.wTree.get_widget("button6").show()
			else:
				self.wTree.get_widget("button6").hide()
		except:
			pass

	def get_proc_info(self):
		try:
			if self.hidecomp == 0:
				telltale = 'Linux version '
				output = commands.getoutput('cat /proc/version')
				inet = output.find(telltale)
				if inet >= 0:
					start = inet + len(telltale)
					end = output.find(' (', start)
					addr = output[start:end]
					self.wTree.get_widget("label3").set_label('Kernel : '+addr)

				output = commands.getoutput('hostname').strip()
				self.wTree.get_widget("label4").set_label(output)
				self.wTree.get_widget("button1").show()
			else:
				self.wTree.get_widget("button1").hide()
		except:
			pass


	def set_ip(self):
		try:
			if self.hidenet == 0:
				devtype = "Network"
				if self.netiface in ["eth0","eth1","eth2","eth3"]:
					devtype = "Wired Network"

				if self.netiface in ["wlan0","rausb0"]:
					devtype = "Wireless Network"

				interface = self.netiface
				telltale = ' ' + interface + '    inet '
				output = commands.getoutput('ip -o addr')

				inet = output.find(telltale,2)
				if inet >= 0:
					start = inet + len(telltale)
					end = output.find('/', start)
					addr = output[start:end]
					self.wTree.get_widget("label7").set_label(addr)
				else:
					self.wTree.get_widget("label7").set_label("Device not configured")

				self.wTree.get_widget("label6").set_label(devtype)
				self.wTree.get_widget("button2").show()
			else:
				self.wTree.get_widget("button2").hide()
		except:
			pass

	def date_time(self):
		try:
			if self.hidedate == 0:
				if self.eudate == 0:
					self.wTree.get_widget("label9").set_label(datetime.datetime.today().strftime("%m-%d-%Y"))
				else:
					self.wTree.get_widget("label9").set_label(datetime.datetime.today().strftime("%d-%m-%Y"))

				self.wTree.get_widget("label10").set_label(datetime.datetime.today().strftime("%H:%M:%S"))
				self.wTree.get_widget("button3").show()
			else:
				self.wTree.get_widget("button3").hide()
		except:
			pass
		return True

	def ButtonClicked( self, widget, event, Exec):
		self.press_x = event.x
		self.press_y = event.y
		self.Exec = Exec

	def ButtonReleased( self, widget, ev, editable ):
		if ev.button == 1:
		        if not hasattr( self, "press_x" ) or \
			        not widget.drag_check_threshold( int( self.press_x ),
			                                        int( self.press_y ),
			                                        int( ev.x ),
			                                        int( ev.y ) ):
			        if self.Exec != "":
					if self.USPWin.pinmenu == False:
					    	self.USPWin.wTree.get_widget( "window1" ).hide()
				    	if "applications" in self.USPWin.plugins:
				    		self.USPWin.plugins["applications"].wTree.get_widget( "entry1" ).grab_focus()
			                Execute( widget, self.Exec )


#---------------------------------------------------------------------------------------#
# USPconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "computer.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "Settings"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/usp/plugins/computer'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/computer', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_FontSizeSpin_changed" : self.fontspin,
				"on_InfoFontSizeSpin_changed" : self.infofontspin,
				"on_WidthSpin_changed" : self.widthspin,
				"on_NetEntry_changed" : self.netchange,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_CompChkBtn_toggled" : self.compchk,
				"on_NetChkBtn_toggled" : self.netchk,
				"on_DateTimeChkBtn_toggled" : self.datetimechk,
				"on_ThemeChkBtn_toggled" : self.themechk,
				"on_ScreenChkBtn_toggled" : self.screenchk,
				"on_ControlChkBtn_toggled" : self.controlchk,
				"on_CompComEntry_changed" : self.compcomchange,
				"on_NetComEntry_changed" : self.netcomchange,
				"on_DateComEntry_changed" : self.datecomchange,
				"on_ThemeComEntry_changed" : self.themecomchange,
				"on_ScrComEntry_changed" : self.scrcomchange,
				"on_ContComEntry_changed" : self.contcomchange,
				"on_CompIconEntry_changed" : self.compiconchange,
				"on_StickyChkBtn_toggled" : self.setsticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf for USPconfig Tab and Set Default Values if they don't exist.
		self.wTree.get_widget("FontSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/computer/settings_font_size',11))
		self.wTree.get_widget("InfoFontSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/computer/settings_info_font_size',9))
		self.wTree.get_widget("WidthSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/computer/width',11))
		self.wTree.get_widget("NetEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/network_interface','eth0'))
		self.wTree.get_widget("HideSepChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_separator', False))
		self.wTree.get_widget("CompChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_computer', False))
		self.wTree.get_widget("NetChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_net', False))
		self.wTree.get_widget("DateTimeChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_date', False))
		self.wTree.get_widget("ThemeChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_theme', False))
		self.wTree.get_widget("ScreenChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_screen', False))
		self.wTree.get_widget("ControlChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/hide_control', False))
		self.wTree.get_widget("CompComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command1','gnome-system-monitor'))
		self.wTree.get_widget("NetComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command2','gksu network-admin'))
		self.wTree.get_widget("DateComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/datetime/system_command1','gksu time-admin'))
		self.wTree.get_widget("ThemeComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command4','gnome-theme-manager'))
		self.wTree.get_widget("ScrComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command5','gnome-display-properties'))
		self.wTree.get_widget("ContComEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/system_command6','gcontrol'))
		self.wTree.get_widget("CompIconEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/computer/icon','gnome-settings.png'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/computer/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def fontspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/computer/settings_font_size',int(self.wTree.get_widget("FontSizeSpin").get_value()))

	def infofontspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/computer/settings_info_font_size',int(self.wTree.get_widget("InfoFontSizeSpin").get_value()))

	def widthspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/computer/width',int(self.wTree.get_widget("WidthSpin").get_value()))

	def netchange(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/computer/network_interface',self.wTree.get_widget("NetEntry").get_text())

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_separator',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_separator',False)

	def compchk(self, widget, **kargs):
		if self.wTree.get_widget("CompChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_computer',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_computer',False)

	def netchk(self, widget, **kargs):
		if self.wTree.get_widget("NetChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_net',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_net',False)

	def datetimechk(self, widget, **kargs):
		if self.wTree.get_widget("DateTimeChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_date',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_date',False)

	def themechk(self, widget, **kargs):
		if self.wTree.get_widget("ThemeChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_theme',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_theme',False)

	def screenchk(self, widget, **kargs):
		if self.wTree.get_widget("ScreenChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_screen',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_screen',False)

	def controlchk(self, widget, **kargs):
		if self.wTree.get_widget("ControlChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/hide_control',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/hide_control',False)

	def compcomchange(self, widget, **kargs):
		self.syscommand1 = self.wTree.get_widget("CompComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/system_command1',self.wTree.get_widget('CompComEntry').get_text())

	def netcomchange(self, widget, **kargs):
		self.syscommand2=self.wTree.get_widget("NetComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/system_command2',self.wTree.get_widget('NetComEntry').get_text())

	def datecomchange(self, widget, **kargs):
		self.syscommand3=self.wTree.get_widget("DateComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/datetime/system_command1',self.wTree.get_widget('DateComEntry').get_text())

	def themecomchange(self, widget, **kargs):
		self.syscommand4=self.wTree.get_widget("ThemeComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/system_command4',self.wTree.get_widget('ThemeComEntry').get_text())

	def scrcomchange(self, widget, **kargs):
		self.syscommand5=self.wTree.get_widget("ScrComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/system_command5',self.wTree.get_widget('ScrComEntry').get_text())

	def contcomchange(self, widget, **kargs):
		self.syscommand6=self.wTree.get_widget("ContComEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/system_command6',self.wTree.get_widget('ContComEntry').get_text())

	def compiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("CompIconEntry").get_text()
		self.client.set_string('/apps/usp/plugins/computer/icon',self.wTree.get_widget('CompIconEntry').get_text())

	def setsticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/computer/sticky',True)
		else:
			self.client.set_bool('/apps/usp/plugins/computer/sticky',False)
