# USP Recent plugin

import gtk
import gtk.glade
import gnome
import sys
import os
import gobject
import datetime
import gconf
import fcntl
import pango
import string
import fnmatch
from execute import Execute
from easygconf import *
from easyfiles import *
from easybuttons import *
from usp_util import *

class pluginclass:
	"""This is the main class for the plugin"""
	"""It MUST be named pluginclass"""

	def __init__( self, USPWin ):

		self.USPWin = USPWin

		#The Glade file for the plugin
		self.gladefile = os.path.join( os.path.dirname( __file__ ), "recent.glade" )

		#Read GLADE file
		self.wTree = gtk.glade.XML( self.gladefile, "window1" )

		self.RecentTips = gtk.Tooltips()

		#These properties are NECESSARY to maintain consistency
		#throughout USP

		#Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget( "window1" )

		#Set 'heading' property for plugin
		self.heading = "Recent"

		#This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget( "eventbox1" )

        	# Items to get custom colors
        	self.itemstocolor = [self.wTree.get_widget( "viewport2" ),self.wTree.get_widget( "viewport1" )]

		#Specify plugin width
		self.width = 180

		#Plugin icon
		self.icon = 'gnome-folder.png'

		self.gconf_dir = '/apps/usp/plugins/recent'
		self.client = gconf.client_get_default()
		self.client.add_dir( '/apps/usp/plugins/recent', gconf.CLIENT_PRELOAD_NONE )
		self.client.notify_add( '/apps/usp/plugins/recent/height', self.RegenPlugin )
		self.client.notify_add( '/apps/usp/plugins/recent/width', self.RegenPlugin )
		self.client.notify_add( '/apps/usp/plugins/recent/recent_font_size', self.RegenPlugin )
		self.client.notify_add( '/apps/usp/plugins/recent/recent_icon_size', self.RegenPlugin )
		self.client.notify_add( '/apps/usp/plugins/recent/num_recent_apps_to_show', self.RegenPlugin)
		self.client.notify_add( '/apps/usp/plugins/recent/num_recent_docs_to_show', self.RegenPlugin)
		self.client.notify_add( '/apps/usp/plugins/recent/hide_tips', self.RegenPlugin)
		self.client.notify_add( '/apps/usp/plugins/recent/hide_document_thumbnails', self.RegenPlugin)
		self.client.notify_add( '/apps/usp/plugins/recent/show_tab_selection', self.RegenPlugin)

		self.thumb_factory = gnome.ui.ThumbnailFactory("normal")
		self.FileList=[]
		self.RecManagerInstance = gtk.recent_manager_get_default()
		self.RecManagerInstance.connect("changed",self.DoRecent)


		self.RegenPlugin()

		#Connect event handlers
		self.wTree.get_widget("ClrBtn").connect("button_press_event",self.clrmenu)
		dic = { "on_combobox1_changed": self.ComboChanged,
				"on_RecentTabs_switch_page": self.TabsChanged}
		self.wTree.signal_autoconnect( dic )

	        self.recent_monitor = FileMonitor(os.path.expanduser("~/.usp/recentapps.list"))
	        self.recent_monitor.open()
	        self.recent_monitor.connect("created", self.do_recent_applications_file)
	        self.recent_monitor.connect("changed", self.do_recent_applications_file)


	def ComboChanged(self, *args, **kargs):
		sel = self.wTree.get_widget( "combobox1" ).get_active()
		self.wTree.get_widget( "RecentTabs" ).set_current_page(sel)

	def TabsChanged(self, *args, **kargs):
		if self.wTree.get_widget( "RecentTabs" ).get_current_page() == 0:
			sel = 1
		else:
			sel = 0
		self.wTree.get_widget( "combobox1" ).set_active(sel)

	def RegenPlugin( self, *args, **kargs ):
		self.GetGconfEntries()
		if self.hidetips == True:
			self.RecentTips.disable()
		else:
			self.RecentTips.enable()
		if self.showtabs == True:
			self.wTree.get_widget('combobox1').hide()
			self.wTree.get_widget('RecentTabs').set_show_tabs(True)
			#self.wTree.get_widget('RecentTabs').set_tab_label_packing(self.wTree.get_widget('label2'),True,True,gtk.PACK_START)
			#self.wTree.get_widget('RecentTabs').set_tab_label_packing(self.wTree.get_widget('label3'),True,True,gtk.PACK_END)
		else:
			self.wTree.get_widget('combobox1').show()
			self.wTree.get_widget('RecentTabs').set_show_tabs(False)

	def GetGconfEntries( self ):
		self.client = gconf.client_get_default()
		self.height = SetGconf( self.client, 'int', '/apps/usp/plugins/recent/height', 385 )
		self.width = SetGconf( self.client, 'int', '/apps/usp/plugins/recent/width', 250 )
		self.numentries = SetGconf( self.client, 'int', '/apps/usp/plugins/recent/num_recent_docs_to_show', 10 )
		self.recentfontsize = SetGconf( self.client, 'int', '/apps/usp/plugins/recent/recent_font_size', 9 )
		self.iconsize = SetGconf( self.client, 'int', '/apps/usp/plugins/recent/recent_icon_size', 1 )
		self.numrecenttoshow = SetGconf( self.client, "int", "/apps/usp/plugins/recent/num_recent_apps_to_show", 10 )
		self.hidetips = SetGconf(self.client,'bool','/apps/usp/plugins/recent/hide_tips', False )
		self.execapp = SetGconf( self.client, "string", '/apps/usp/plugins/applications/execute_app', "nautilus" )
		self.showappsfirst = SetGconf(self.client,'bool','/apps/usp/plugins/recent/show_apps_on_start', False )
		self.hide_thumbs = SetGconf(self.client,'bool','/apps/usp/plugins/recent/hide_document_thumbnails', False)
		self.showtabs = SetGconf(self.client,'bool','/apps/usp/plugins/recent/show_tab_selection', False)

		# Hide vertical dotted separator
		self.hideseparator = SetGconf( self.client, "bool", "/apps/usp/plugins/recent/hide_separator", False )
		# Plugin icon
		self.icon = SetGconf( self.client, "string", '/apps/usp/plugins/recent/icon', "gnome-fs-directory" )
		# Allow plugin to be minimized to the left plugin pane
		self.sticky = SetGconf( self.client, "bool", "/apps/usp/plugins/recent/sticky", False )
		self.minimized = SetGconf( self.client, "bool", "/apps/usp/plugins/recent/minimized", False )
		self.RebuildPlugin()

	def SetHidden( self, state ):
		if state == True:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/recent/minimized", True )
		else:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/recent/minimized", False )


	def RebuildPlugin(self):
		self.content_holder.set_size_request(self.width, self.height )
		self.do_recent_applications_file()
		self.DoRecent()
		if self.showappsfirst == True:
			self.wTree.get_widget( "combobox1" ).set_active(0)
			self.wTree.get_widget( "RecentTabs" ).set_current_page(0)
		else:
			self.wTree.get_widget( "combobox1" ).set_active(1)
			self.wTree.get_widget( "RecentTabs" ).set_current_page(1)

	def DoRecent( self, *args, **kargs ):
		try:
			for i in self.wTree.get_widget( "RecentBox" ).get_children():
			        i.destroy()
		except:
			pass
		try:
			self.wTree.get_widget( "vbox1" ).set_size_request( self.width, self.height )
		except:
			pass

		if len( self.wTree.get_widget( "RecentBox" ).get_children() ) < self.numentries:
			n=len( self.wTree.get_widget( "RecentBox" ).get_children() )-1
		else:
			n=self.numentries-1
		while n >= 0:
			self.wTree.get_widget( "RecentBox" ).remove( self.wTree.get_widget( "RecentBox" ).get_children()[n] )
			n-=1

		self.FileList, self.UnFilteredFileList, self.IconList = self.GetRecent()
		loc = 0
		for Name in self.FileList:
			if Name != None:
						 self.AddRecentBtn( Name, self.UnFilteredFileList[loc], self.IconList[loc] )
			loc = loc + 1
		return True

	def clrmenu(self, widget, event):
		sel = self.wTree.get_widget( "combobox1" ).get_active()
		if event.button == 1:
			self.do_clrmenu(sel)
		elif event.button == 3:
			self.do_clrmenu(0)
			self.do_clrmenu(1)

	def do_clrmenu(self,selected):
		if selected == 0:
			recentlistpath = os.path.expanduser( "~" ) + "/.usp/recentapps.list"
			out_file = open( recentlistpath, "w" )
			out_file.write( "" )
			out_file.close()

			self.do_recent_applications_file()
		if selected == 1:
			self.RecManagerInstance.purge_items()

			self.DoRecent()

	def AddRecentBtn( self, Name, UnFilt, RecentImage ):
		if self.iconsize >= 4:
			icon_size = gtk.ICON_SIZE_DIALOG
			icon_scale = 48
		elif self.iconsize == 3:
			icon_size = gtk.ICON_SIZE_DND
			icon_scale = 32
		elif self.iconsize == 2:
			icon_size = gtk.ICON_SIZE_BUTTON
			icon_scale = 24
		elif self.iconsize <= 1:
			icon_size = gtk.ICON_SIZE_MENU
			icon_scale = 16

		DispName=os.path.basename( Name )
		AButton = gtk.Button( "", "ok", True )
		AButton.remove( AButton.get_children()[0] )
		AButton.set_size_request( self.width-28, -1 )
		AButton.set_relief( gtk.RELIEF_NONE )
		AButton.connect( "button_press_event", self.callback, Name )
		AButton.connect( "button_release_event", self.ButtonReleased, True, False )
		#Use the Unfiltered Filename for the ButtonName for use with DelButton() to id which URI to remove.
		AButton.MyButtonName = UnFilt

		Align1 = gtk.Alignment( 0, 0.5, 0, 0 )
		Align1.set_padding( 0, 0, 0, 0 )
		HBox1 = gtk.HBox( False, 2 )
		VBox1 = gtk.VBox( False, 2 )

		VBox1.show()


		Label1 = gtk.Label( DispName )
		Label1.set_size_request( AButton.size_request()[0]-20, -1 )
		Label1.set_ellipsize( pango.ELLIPSIZE_END )
		Label1.set_alignment( 0.0, 0.0 )
		Label1.show()
		LStyle = pango.AttrList()
		attr = pango.AttrSize(self.recentfontsize*1000,0,-1)
		LStyle.insert(attr)
		Label1.set_attributes(LStyle)
		VBox1.add( Label1 )

		if self.hide_thumbs == False:
			ButtonIcon = icon_factory.load_image( RecentImage, icon_size )
			ButtonIcon.set_size_request( icon_scale, -1 )
			HBox1.add( ButtonIcon )
			ButtonIcon.show()

		HBox1.add( VBox1 )
		HBox1.show()
		Align1.add( HBox1 )
		Align1.show()
		AButton.add( Align1 )
		AButton.show()

		self.RecentTips.set_tip( AButton, Name, tip_private=None )

		self.wTree.get_widget( "RecentBox" ).pack_start( AButton, False, True, 0 )

	def callback(self, widget, event, Exec):
		self.press_x = event.x
		self.press_y = event.y
		self.Exec = Exec
		self.USPWin.wTree.get_widget( "window1" ).hide()

		x = os.system("gnome-open \""+Exec+"\"")
		if x == 256:
			dia = gtk.Dialog('File not found!',
			                 None,  #the toplevel wgt of your app
			                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,  #binary flags or'ed together
			                 ("Ok", gtk.RESPONSE_OK))
			dia.vbox.pack_start(gtk.Label(Exec+' could not be found!'))
			dia.vbox.show_all()
			dia.show()
			result = dia.run()
			if result == gtk.RESPONSE_OK:
				dia.destroy()
		
		

	def GetRecent(self, *args, **kargs):
		icon_theme = gtk.icon_theme_get_default()
		FileString=[]
		UnFilteredFileString=[]
		IconString=[]
		RecentInfo=self.RecManagerInstance.get_items()
		count=0
		MaxEntries=self.numentries
		if self.numentries == -1:
			MaxEntries=len(RecentInfo)
		for items in RecentInfo:
			FileString.append(items.get_uri_display())
			UnFilteredFileString.append(items.get_uri())
			if self.hide_thumbs == False:
				icon_name = gnome.ui.icon_lookup(icon_theme, self.thumb_factory, items.get_uri(), items.get_mime_type(), 4)[0]
			else:
				icon_name = ""
			IconString.append ( icon_name )			
			#IconString.append( items.get_icon(gtk.ICON_SIZE_DIALOG) )
			count+=1
			if count >= MaxEntries:
				break
		return FileString, UnFilteredFileString, IconString


	def do_recent_applications_file( self, *args, **kargs ):
	    TestForFiles( ".usp/recentapps.list" )


	    RecentapplicationsFile = open ( os.path.join( os.path.expanduser( "~" ), ".usp/recentapps.list" ), "r" )
	    RecentapplicationsList = RecentapplicationsFile.readlines()
	    RecentapplicationsList.reverse()

	    try:
	    	for i in self.wTree.get_widget( "RecentApps" ).get_children():
	        	i.destroy()
	    except:
	    	pass

	    if len( RecentapplicationsList ) < 1:
	        print "No recent applications"
	    else:

	    	if self.numrecenttoshow >= len( RecentapplicationsList ) or self.numrecenttoshow == -1:
	    		recentdepth = len( RecentapplicationsList )
	    	else:
	    		recentdepth = self.numrecenttoshow
	        for i in range( recentdepth ):
	            if ( RecentapplicationsList[i][-8:-1] == 'desktop' ):
	                RecentApplicationData = ParseDesktopFile( RecentapplicationsList[i] )

	                if len( RecentApplicationData ) > 1 and RecentApplicationData[0] != '':
	                    InTerm=False
	                    if RecentApplicationData[5].upper()=="TRUE":
	                        InTerm=True
	                    Button1 = MakeAButton(self.width-28, -1, gtk.RELIEF_NONE, RecentApplicationData[3], self.iconsize, 48, -1, [RecentApplicationData[0]], [[pango.AttrWeight( pango.WEIGHT_NORMAL, 0, -1 ), pango.AttrSize( self.recentfontsize*1000, 0, -1 )], [""]] )
	                    Button1.connect( "button_press_event", self.ButtonClicked, RecentApplicationData[2].split() )
	                    Button1.connect( "button_release_event", self.ButtonReleased, True, InTerm )
	                    self.RecentTips.set_tip( Button1, RecentApplicationData[1], tip_private=None )
	                    Button1.MyButtonName = RecentapplicationsList[i][:-1]
	                    try:
	                        self.wTree.get_widget( "RecentApps" ).pack_start( Button1, False, False, 10 )
	                    except:
	                        pass

	def DelButton( self, a, widget, itemclicked, event, menutogo, *args ):
		sel = self.wTree.get_widget( "combobox1" ).get_active()
		if sel==0:
			self.wTree.get_widget( "RecentApps" ).remove(widget)
			TestForFiles( ".usp/recentapps.list" )
			RecentapplicationsFile = open ( os.path.join( os.path.expanduser( "~" ), ".usp/recentapps.list" ), "r" )
			RecentapplicationsList = RecentapplicationsFile.readlines()
			RecentapplicationsFile.close()
			if widget.MyButtonName+"\n" in RecentapplicationsList:
				RecentapplicationsList.remove(widget.MyButtonName+"\n")
			RecentapplicationsFile = open ( os.path.join( os.path.expanduser( "~" ), ".usp/recentapps.list" ), "w" )
			RecentapplicationsFile.writelines(RecentapplicationsList)
			RecentapplicationsFile.close()

			self.do_recent_applications_file()
		elif sel==1:
			self.RecManagerInstance.remove_item(widget.MyButtonName)
			self.DoRecent()

	def OpenDir( self, a, widget, itemclicked, event, menutogo, *args ):
		foldername = os.path.dirname( widget.MyButtonName )
		try:
			os.system( self.execapp +" "+ foldername )
		except:
			pass
		if self.USPWin.pinmenu == False:
		    	self.USPWin.wTree.get_widget( "window1" ).hide()
    		if "applications" in self.USPWin.plugins:
   			self.USPWin.plugins["applications"].wTree.get_widget( "entry1" ).grab_focus()
		return

	def ButtonClicked( self, widget, event, Exec ):
		self.press_x = event.x
		self.press_y = event.y
		self.Exec = Exec

	def ButtonReleased( self, w, ev, ev2, InTerm=False ):
		if ev.button == 1:
			if not hasattr( self, "press_x" ) or \
				not w.drag_check_threshold( int( self.press_x ),
											 int( self.press_y ),
											 int( ev.x ),
											 int( ev.y ) ):
				WriteRecentListFile( ".usp/recentapps.list", w.MyButtonName, "a" )
				if self.USPWin.pinmenu == False:
				    	self.USPWin.wTree.get_widget( "window1" ).hide()
		    		if "applications" in self.USPWin.plugins:
		   			self.USPWin.plugins["applications"].wTree.get_widget( "entry1" ).grab_focus()
				Execute( w, self.Exec, InTerm )
		elif ev.button == 3:
			itemclicked = w
			self.mTree = gtk.glade.XML( self.gladefile, "menu1" )
			x = int( ev.x )
			y = int( ev.y )
			time = ev.time
			self.mTree.get_widget( "RemoveItem" ).connect( "activate", self.DelButton, w, itemclicked, ev, self.mTree )
			sel = self.wTree.get_widget( "combobox1" ).get_active()
			if sel==1:
				self.mTree.get_widget( "OpenFolder" ).show()
				self.mTree.get_widget( "OpenFolder" ).connect( "activate", self.OpenDir, w, itemclicked, ev, self.mTree )
				self.mTree.get_widget( "FavItem" ).hide()

			else:
				self.mTree.get_widget( "OpenFolder" ).hide()
				self.mTree.get_widget( "FavItem" ).show()
				self.mTree.get_widget( "FavItem" ).connect( "activate", self.AddFavourite, itemclicked, self.mTree )
			self.mTree.get_widget( "menu1" ).popup( None, None, None, ev.button, time )

	def AddToFavourites( self, location ):
		#Quick work around for not rewriting all the code :-(
		location = location + "  "

		ApplicationData = ""
		ButtonIcon = None
		# For Folders and Network Shares
		ButtonLocation = string.join( location[:-2].split( "%20" ) )
		ButtonCmd = self.execapp + ' ' + location[:-2]

		if ButtonLocation[:4] == 'file':
			ButtonIcon = "gnome-fs-directory"

		if ButtonLocation[:3] == 'smb' or ButtonLocation[:3] == 'ssh' or ButtonLocation[:7] == 'network':
			ButtonIcon = "gnome-fs-network"

		if ButtonIcon != None:
			locationName = [ButtonLocation.split( '/' )[-1:][0]]
			if locationName == [""]:
				locationName = "/"
			ApplicationData = [locationName[0], '', ButtonCmd, ButtonIcon, '', 'False']

		# For .desktop files
		if location[-9:-2] == "desktop":
			ApplicationData = ParseDesktopFile( location[:-2] )

		#For Special locations
		if location[:-2] == 'x-nautilus-desktop:///computer':
			ApplicationData = ParseDesktopFile( "/usr/share/applications/nautilus-computer.desktop" )

		if location[:-2] == 'x-nautilus-desktop:///home':
			ApplicationData = ParseDesktopFile( "/usr/share/applications/nautilus-home.desktop" )

		if location[:-2] == 'x-nautilus-desktop:///network':
			ApplicationData = ParseDesktopFile( "/usr/share/applications/network-scheme.desktop" )

		if len( ApplicationData ) > 1:
			#Button1 = MakeAButton( self.width-10, -1, gtk.RELIEF_NONE, ApplicationData[3], gtk.ICON_SIZE_BUTTON, 48, -1, [ApplicationData[0]], [""] )
			#Button1.connect( "button_press_event", self.ButtonClicked, ApplicationData[2].split() )
			#InTerm=False
			#if ApplicationData[5].upper()=="TRUE":
			#	InTerm=True
			#Button1.connect( "button_release_event", self.ButtonReleased, True, InTerm )
			#self.wTree.get_widget( "applications_button_holder" ).pack_start( Button1, False, False, 10 )

			FileExists = False
			FileData = GetFilePath( location )
			clean1 = FileData.replace( "/", "-" )
			DroppedFile = clean1.replace( " ", "" ) #+".desktop"
			# Next section just filters out if it's in the local apps folder already.
			split1 = DroppedFile.split("--")
			if len(split1) > 1:
				DroppedFile = "-"+split1[1]
			else:
				DroppedFile = split1[0]

			for fileName in os.listdir ( os.path.join( os.path.expanduser( "~" ), ".usp/applications" ) ):
				if fnmatch.fnmatch ( fileName, DroppedFile ):
					FileExists = True

			if FileExists == True:
				print "File Exists!"
				#Button1.destroy()
			else:
				DroppedFile = ".usp/applications/" + DroppedFile
				DroppedFile = os.path.join( os.path.expanduser( "~" ), DroppedFile )
				print "Application added - " + DroppedFile
				#Button1.MyButtonName = DroppedFile
				WriteDesktopFile( DroppedFile, ApplicationData, ".usp/applications.list" )

	def AddFavourite( self, widget, button, menutogo ):
		self.AddToFavourites( button.MyButtonName )
		menutogo.get_widget( "menu1" ).destroy()

	def do_plugin(self):
		self.DoRecent()
		self.do_recent_applications_file()


#---------------------------------------------------------------------------------------#
# USPconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "recent.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "Recent"
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/usp/plugins/recent'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/recent', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_RecFontSizeSpin_changed" : self.recfontspin,
				"on_RecIconSizeSpin_changed" : self.reciconspin,
				"on_HideThumbsChkBtn_toggled" : self.hidethumbschk,
				"on_RecNumSpin_changed" : self.recnumspin,
				"on_RecAppNumSpin_changed" : self.recappspin,
				"on_RecHSpin_changed" : self.rechspin,
				"on_RecWSpin_changed" : self.recwspin,
				"on_TipsChkBtn_toggled" : self.tipschk,
				"on_ShowAppsChkBtn_toggled" : self.showappschk,
				"on_ShowTabsChkBtn_toggled" : self.showtabschk,
				"on_RecIconEntry_changed" : self.reciconchange,
				"on_StickyChkBtn_toggled" : self.setsticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from GConf for USPconfig Tab and Set Default Values in GConf if they don't exist.
		self.wTree.get_widget("RecFontSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/recent_font_size',9))
		self.wTree.get_widget("RecIconSizeSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/recent_icon_size',1))
		self.wTree.get_widget("HideThumbsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/recent/hide_document_thumbnails',False))
		self.wTree.get_widget("RecNumSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/num_recent_docs_to_show',10))
		self.wTree.get_widget("RecAppNumSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/num_recent_apps_to_show',10))
		self.wTree.get_widget("RecHSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/height',200))
		self.wTree.get_widget("RecWSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/recent/width',250))
		self.wTree.get_widget("TipsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/recent/hide_tips',True))
		self.wTree.get_widget("ShowAppsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/recent/show_apps_on_start',False))
		self.wTree.get_widget("ShowTabsChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/recent/show_tab_selection',False))
		self.wTree.get_widget("RecIconEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/recent/icon','gnome-folder.png'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/recent/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def recfontspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/recent_font_size',int(self.wTree.get_widget("RecFontSizeSpin").get_value()))

	def reciconspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/recent_icon_size',int(self.wTree.get_widget("RecIconSizeSpin").get_value()))

	def hidethumbschk(self, widget, **kargs):
		if self.wTree.get_widget("HideThumbsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/recent/hide_document_thumbnails',True)
		else:
			self.client.set_bool('/apps/usp/plugins/recent/hide_document_thumbnails',False)

	def recnumspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/num_recent_docs_to_show',int(self.wTree.get_widget("RecNumSpin").get_value()))

	def recappspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/num_recent_apps_to_show',int(self.wTree.get_widget("RecAppNumSpin").get_value()))

	def rechspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/height',int(self.wTree.get_widget("RecHSpin").get_value()))

	def recwspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/recent/width',int(self.wTree.get_widget("RecWSpin").get_value()))

	def tipschk(self, widget, **kargs):
		if self.wTree.get_widget("TipsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/recent/hide_tips',True)
		else:
			self.client.set_bool('/apps/usp/plugins/recent/hide_tips',False)

	def showappschk(self, widget, **kargs):
		if self.wTree.get_widget("ShowAppsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/recent/show_apps_on_start',True)
		else:
			self.client.set_bool('/apps/usp/plugins/recent/show_apps_on_start',False)

	def showtabschk(self, widget, **kargs):
		if self.wTree.get_widget("ShowTabsChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/recent/show_tab_selection',True)
		else:
			self.client.set_bool('/apps/usp/plugins/recent/show_tab_selection',False)

	def reciconchange(self, widget, **kargs): 
		self.icon=self.wTree.get_widget("RecIconEntry").get_text()
		self.client.set_string('/apps/usp/plugins/recent/icon',self.wTree.get_widget('RecIconEntry').get_text())

	def setsticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/recent/sticky',True)
		else:
			self.client.set_bool('/apps/usp/plugins/recent/sticky',False)


