#!/usr/bin/env python
# USP System Management plugin

import gtk
import gtk.glade
import sys
import os
import gobject
import gconf
import gnomevfs
import string
from easyfiles import *
from easybuttons import *
from execute import *
from easygconf import *

class pluginclass:
	def __init__(self, USPWin):

		self.USPWin = USPWin
		# The Glade file for the plugin
		self.gladefile = os.path.join(os.path.dirname(__file__), "system_management.glade")

		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile, "window1")

		self.ManagementTips = gtk.Tooltips()

		# These properties are NECESSARY to maintain consistency
		# throughout USP

		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("window1")
		#dic = {"on_window1_destroy" : gtk.main_quit}
		#self.wTree.signal_autoconnect(dic)

		# Set 'heading' property for plugin
		self.heading = "System Management"

		self.icon = "gnome-system"

		# This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget("eventbox1")

		# Items to get custom colors
		self.itemstocolor = [self.wTree.get_widget("eventbox1")]

		# Gconf stuff
		self.gconf_dir = '/apps/usp/plugins/system-management'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/system-management', gconf.CLIENT_PRELOAD_NONE)
		self.client.notify_add( '/apps/usp/plugins/system-management/font_size', self.RegenPlugin )
		self.client.notify_add( '/apps/usp/plugins/system-management/icon_size', self.RegenPlugin )
		self.width = SetGconf(self.client,"int","/apps/usp/plugins/system-management/width",200)
		self.height = SetGconf(self.client,"int","/apps/usp/plugins/system-management/height",130)
		self.installsoftware = SetGconf(self.client,"string","/apps/usp/plugins/system-management/install_software_cmd", "gksu /usr/sbin/synaptic").split()
		self.controlcenter = SetGconf(self.client,"string","/apps/usp/plugins/system-management/control_center_cmd","gcontrol").split()
		self.lockscreen = SetGconf(self.client,"string","/apps/usp/plugins/system-management/lock_screen_cmd","gnome-screensaver-command --lock").split()
		self.quit_session = SetGconf(self.client,"string","/apps/usp/plugins/system-management/quit_cmd", "gnome-session-save --kill").split()
		self.hideseparator = SetGconf(self.client,"bool","/apps/usp/plugins/system-management/hide_separator", False)
		self.fontsize = SetGconf(self.client,'int','/apps/usp/plugins/system-management/font_size',9)
		self.iconsize = SetGconf(self.client,'int','/apps/usp/plugins/system-management/icon_size',2)

		# Allow plugin to be minimized to the left plugin pane
		self.sticky = SetGconf(self.client,"bool","/apps/usp/plugins/system-management/sticky",False)
		self.minimized = SetGconf( self.client, "bool", "/apps/usp/plugins/system-management/minimized", False )

		# Plugin icon
		self.icon = SetGconf(self.client,"string",'/apps/usp/plugins/system-management/icon',"gnome-system")

	def RegenPlugin( self, *args, **kargs ):
		self.do_plugin()

	def SetHidden( self, state ):
		if state == True:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/system-management/minimized", True )
		else:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/system-management/minimized", False )


	def ButtonClicked(self,widget, event, Exec):
		self.press_x = event.x
		self.press_y = event.y
		self.Exec = Exec

	def ButtonReleased(self, w, ev):
		if ev.button == 1:
			if not hasattr(self, "press_x") or \
				not w.drag_check_threshold(int(self.press_x),
											 int(self.press_y),
											 int(ev.x),
											 int(ev.y)):
		    		if self.USPWin.pinmenu == False:
		    			self.USPWin.wTree.get_widget( "window1" ).hide()
		    		if "applications" in self.USPWin.plugins:
		    			self.USPWin.plugins["applications"].wTree.get_widget( "entry1" ).grab_focus()
				Execute(w,self.Exec)


	def do_sizing(self):
		if self.iconsize == 4 or self.iconsize > 4:
			icon_size = gtk.ICON_SIZE_DIALOG
		elif self.iconsize == 3:
			icon_size = gtk.ICON_SIZE_DND
		elif self.iconsize == 2:
			icon_size = gtk.ICON_SIZE_BUTTON
		elif self.iconsize == 1 or self.iconsize < 1:
			icon_size = gtk.ICON_SIZE_MENU

		self.wTree.get_widget("install_software").child.get_children()[0].set_property("icon-size", icon_size)
		self.wTree.get_widget("control_center").child.get_children()[0].set_property("icon-size", icon_size)
		self.wTree.get_widget("lock_screen").child.get_children()[0].set_property("icon-size", icon_size)
		self.wTree.get_widget("quit_session").child.get_children()[0].set_property("icon-size", icon_size)

		LStyle = pango.AttrList()
		attr = pango.AttrSize(self.fontsize*1000,0,-1)
		LStyle.insert(attr)
		attr = pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1)
		LStyle.insert(attr)
		self.wTree.get_widget("install_software").child.get_children()[1].set_attributes(LStyle)
		self.wTree.get_widget("control_center").child.get_children()[1].set_attributes(LStyle)
		self.wTree.get_widget("lock_screen").child.get_children()[1].set_attributes(LStyle)
		self.wTree.get_widget("quit_session").child.get_children()[1].set_attributes(LStyle)

	def do_setup_window(self):
		self.wTree.get_widget("install_software").connect("button_press_event",self.ButtonClicked,self.installsoftware)
		self.wTree.get_widget("install_software").connect("button_release_event",self.ButtonReleased)
		self.ManagementTips.set_tip( self.wTree.get_widget("install_software"), 'Install, remove and upgrade software packages', tip_private=None )
		self.wTree.get_widget("control_center").connect("button_press_event",self.ButtonClicked,self.controlcenter)
		self.wTree.get_widget("control_center").connect("button_release_event",self.ButtonReleased)
		self.ManagementTips.set_tip( self.wTree.get_widget("control_center"), 'Control center to manage all your settings', tip_private=None )
		self.wTree.get_widget("lock_screen").connect("button_press_event",self.ButtonClicked,self.lockscreen)
		self.wTree.get_widget("lock_screen").connect("button_release_event",self.ButtonReleased)
		self.ManagementTips.set_tip( self.wTree.get_widget("lock_screen"), 'Lock your computer', tip_private=None )
		self.wTree.get_widget("quit_session").connect("button_press_event",self.ButtonClicked,self.quit_session)
		self.wTree.get_widget("quit_session").connect("button_release_event",self.ButtonReleased)
		self.ManagementTips.set_tip( self.wTree.get_widget("quit_session"), 'Log off, switch user, lock screen or power down the computer', tip_private=None )

	def do_plugin(self):
		self.do_sizing()
		self.do_setup_window()

#---------------------------------------------------------------------------------------#
# USPconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "system_management.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "System Management"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/usp/plugins/system-management'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/system-management', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_InstCmdEntry_changed" : self.instchange,
				"on_ContCmdEntry_changed" : self.contchange,
				"on_LockCmdEntry_changed" : self.lockchange,
				"on_QuitCmdEntry_changed" : self.quitchange,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_SysFontSizeSpin_changed" : self.sysfontspin,
				"on_SysIconSizeSpin_changed" : self.sysiconspin,
				"on_SysHSpin_changed" : self.sysh,
				"on_SysWSpin_changed" : self.sysw,
				"on_SysIconEntry_changed" : self.sysiconchange,
				"on_StickyChkBtn_toggled" : self.syssticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("InstCmdEntry").set_text(SetGconf(self.client,"string","/apps/usp/plugins/system-management/install_software_cmd", "gksu /usr/sbin/synaptic"))
		self.wTree.get_widget("ContCmdEntry").set_text(SetGconf(self.client,"string","/apps/usp/plugins/system-management/control_center_cmd","gcontrol"))
		self.wTree.get_widget("LockCmdEntry").set_text(SetGconf(self.client,"string","/apps/usp/plugins/system-management/lock_screen_cmd","gnome-screensaver-command --lock"))
		self.wTree.get_widget("QuitCmdEntry").set_text(SetGconf(self.client,"string","/apps/usp/plugins/system-management/quit_cmd", "gnome-session-save --kill"))
		self.wTree.get_widget("HideSepChkBtn").set_active(SetGconf(self.client, 'bool', '/apps/usp/plugins/system-management/hide_separator',False))
		self.wTree.get_widget("SysFontSizeSpin").set_value(int(SetGconf(self.client,'int','/apps/usp/plugins/system-management/font_size',9)))
		self.wTree.get_widget("SysIconSizeSpin").set_value(int(SetGconf(self.client,'int','/apps/usp/plugins/system-management/icon_size',2)))
		self.wTree.get_widget("SysHSpin").set_value(int(SetGconf(self.client,'int','/apps/usp/plugins/system-management/height',280)))
		self.wTree.get_widget("SysWSpin").set_value(int(SetGconf(self.client,'int','/apps/usp/plugins/system-management/width',200)))
		self.wTree.get_widget("SysIconEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/system-management/icon','gnome-system'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/system-management/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def instchange(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/system-management/install_software_cmd',self.wTree.get_widget("InstCmdEntry").get_text())

	def contchange(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/system-management/control_center_cmd',self.wTree.get_widget("ContCmdEntry").get_text())

	def lockchange(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/system-management/lock_screen_cmd',self.wTree.get_widget("LockCmdEntry").get_text())

	def quitchange(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/system-management/quit_cmd',self.wTree.get_widget("QuitCmdEntry").get_text())

	def sysfontspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/system-management/font_size',int(self.wTree.get_widget("SysFontSizeSpin").get_value()))

	def sysiconspin(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/system-management/icon_size',int(self.wTree.get_widget("SysIconSizeSpin").get_value()))

	def sysh(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/system-management/height',int(self.wTree.get_widget("SysHSpin").get_value()))

	def sysw(self, widget, **kargs):
		self.client.set_int('/apps/usp/plugins/system-management/width',int(self.wTree.get_widget("SysWSpin").get_value()))

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/system-management/hide_separator',True)
		else:
			self.client.set_bool('/apps/usp/plugins/system-management/hide_separator',False)

	def sysiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("SysIconEntry").get_text()
		self.client.set_string('/apps/usp/plugins/system-management/icon',self.wTree.get_widget('SysIconEntry').get_text())

	def syssticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/system-management/sticky',True)
		else:
			self.client.set_bool('/apps/usp/plugins/system-management/sticky',False)

