# User USP plugin

import gtk
import gtk.glade
import sys
import os
import gobject
import datetime
import gconf
import pango
import commands
import getpass
import pwd

from execute import Execute
from easygconf import *
from easyfiles import *
from easybuttons import *
from usp_util import *

class pluginclass:
	"""This is the main class for the plugin"""
	"""It MUST be named pluginclass"""
	def __init__( self, USPWin ):

		self.USPWin = USPWin

		#The Glade file for the plugin
		self.gladefile = os.path.join( os.path.dirname( __file__ ), "uspuser.glade" )

		#Read GLADE file
		self.wTree = gtk.glade.XML( self.gladefile, "window1" )

		#These properties are NECESSARY to maintain consistency
		#throughout USP

		#Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("window1")

		#Set 'heading' property for plugin
		self.heading = "User"

		#This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget( "eventbox1" )

		#Specify plugin width
		self.width = 150

		#Plugin icon
		self.icon = 'system-users'

		self.gconf_dir = '/apps/usp/plugins/user'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/user', gconf.CLIENT_PRELOAD_NONE)
		self.client.notify_add('/apps/usp/plugins/user/settings_font_size',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/settings_font_bold',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/name_below',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/hide_logged_on',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/hide_user_name',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/show_full_name', self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/animated_picture', self.RegenPlugin )
		self.client.notify_add('/apps/usp/plugins/datetime/settings_date_sep',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/datetime/eu_date_format',self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/height', self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/width', self.RegenPlugin)
		self.client.notify_add('/apps/usp/plugins/user/icon', self.RegenPlugin)

		self.face_monitor = FileMonitor(os.path.expanduser("~/.face"))
		self.face_monitor.open()
		self.face_monitor.connect("created", self.showuser)
		self.face_monitor.connect("changed", self.showuser)

		self.RegenPlugin()
		#gobject.timeout_add(1000, self.showuser)

	def RegenPlugin(self, *args, **kargs):
		self.GetGconfEntries()
		self.SetSystemStyle(["UserLabel","UserBelowLabel"])
		self.RebuildPlugin()
		self.showuser()

	def RebuildPlugin(self):
		self.content_holder.set_size_request(self.width, self.height )

	def SetupEvents(self):
		#Connect event handlers
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_ChooserBtn_clicked" : self.FaceChooser}
		self.wTree.signal_autoconnect(dic)

	def GetGconfEntries(self):
		self.client = gconf.client_get_default()
		self.fontsize = SetGconf(self.client,'int','/apps/usp/plugins/user/settings_font_size',14)
		self.fontbold = SetGconf(self.client,'bool','/apps/usp/plugins/user/settings_font_bold', False)
		self.namebelow = SetGconf(self.client,'bool','/apps/usp/plugins/user/name_below', False)
		self.hidelogged = SetGconf(self.client,'bool','/apps/usp/plugins/user/hide_logged_on', False)
		self.hidename = SetGconf(self.client,'bool','/apps/usp/plugins/user/hide_user_name', False)
		self.fullname = SetGconf(self.client,'bool','/apps/usp/plugins/user/show_full_name', False)
		self.useranim=SetGconf(self.client,'string','/apps/usp/plugins/user/animated_picture','')
		self.datesep = SetGconf(self.client,'string','/apps/usp/plugins/datetime/settings_date_sep','-')
		self.eudate = SetGconf(self.client,'bool','/apps/usp/plugins/datetime/eu_date_format', False)
		self.width = SetGconf( self.client, "int", "/apps/usp/plugins/user/width", 180 )
		self.height = SetGconf( self.client, "int", "/apps/usp/plugins/user/height", 200 )
		self.icon = SetGconf( self.client, "string", "/apps/usp/plugins/user/icon", 'login-photo' )
		# Allow plugin to be minimized to the left plugin pane
		self.sticky = SetGconf( self.client, "bool", "/apps/usp/plugins/user/sticky", False )
		self.minimized = SetGconf( self.client, "bool", "/apps/usp/plugins/user/minimized", False )

	def SetHidden( self, state ):
		if state == True:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/user/minimized", True )
		else:
			WriteGconf( self.client, "bool", "/apps/usp/plugins/user/minimized", False )

	def FaceChooser(self, widget):
		os.system('gdmphotosetup &')
		self.USPWin.wTree.get_widget( "window1" ).hide()
		self.showuser()
        
	def SetSystemStyle(self,items):
		# Set Font styles
		HeadingStyle = pango.AttrList()
		attr = pango.AttrSize(self.fontsize*1000,0,-1)
		HeadingStyle.insert(attr)
		if self.fontbold == 0:
			attr = pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1)
		else:
			attr = pango.AttrWeight(pango.WEIGHT_BOLD,0,-1)
		HeadingStyle.insert(attr)

		for i in range(len(items)):
			self.wTree.get_widget(items[i]).set_attributes(HeadingStyle)

	def timetosecs(s):
		hms = s.split(":") # [hh, mm, ss]
		secs = 0
		for t in hms:
			secs = secs * 60 + int(t)
		return secs

	def secstotime(secs):
		hms = []
		while secs:
			hms.append(str(secs % 60))
			secs = secs // 60
		return ":".join(hms)

	#Actually do something with the plugin
	def showuser(self, *args, **kargs):
		if self.useranim == '':
			facepic = os.path.join(os.path.expanduser("~"), '.face')
		else:
			facepic=self.useranim
		try:
			self.anim=gtk.gdk.PixbufAnimation(facepic)
			if self.anim.is_static_image()==False:
				print "Non Static User Image Used"
				self.animiter=self.anim.get_iter(0.0)
				self.animiter.advance()
			else:
				print "Static User Image Used"
			self.wTree.get_widget('Face').set_from_file(facepic)
		except:
			print "No User Picture Specified or File Not Found"
			pass
		user = pwd.getpwnam(getpass.getuser())
		# Dummy value if not displayed
		lastlogtime = ["0000-00-00","00:00"]
		# Only need to do this section if logged time is to be displayed
		if self.hidelogged == False:
			who_out = commands.getoutput('who -u')
			if self.USPWin.VERBOSE == True:
				print "Output User Array\n",who_out
			try:
				wholist = who_out.split("\n")
			except:
				# Construct array in case of error
				print "Error obtaining information from 'who'"
				wholist=[["Error", "", "0000-00-00","00:00"],["Error", "", "0000-00-00","00:00"]]
				pass
			# This is an Extra Check that sometimes occurs from 'who' erroring.
			if len(who_out) == 0:
				print "Error obtaining information from 'who'"
				wholist=[["Error", "", "0000-00-00","00:00"],["Error", "", "0000-00-00","00:00"]]
			if self.USPWin.VERBOSE == True:
				print "Output User Array\n", who_out
			individualDict = dict((ind.split()[0], ind) for ind in wholist)
			match = individualDict[user[0]]
			lastlogtime = match.split()[2],match.split()[3]
		# This is an Extra Check.
		if len(user) == 0:
			print "Error obtaining information from 'user'"
			userlist=["Error", "", "0000-00-00","00:00"]
		if self.fullname == True:
			username = user[4].split(',')[0]
		else:
			username = user[0]
		userlist=[username, "", lastlogtime[0],lastlogtime[1]]

		datelogged = userlist[2]
		if self.eudate == 0:
			showdate=datelogged[5:7]+self.datesep+datelogged[8:10]+self.datesep+datelogged[0:4]
		else:
			showdate=datelogged[8:10]+self.datesep+datelogged[5:7]+self.datesep+datelogged[0:4]
		# Show User
		self.wTree.get_widget('UserLabel').set_label(userlist[0])
		self.wTree.get_widget('UserBelowLabel').set_label(userlist[0])
		# Show User Date Logged On
		self.wTree.get_widget('DateLabel').set_label("Logon Date : "+showdate)
		self.wTree.get_widget('DateBelowLabel').set_label("Logon Date : "+showdate)
		# Show User Time Logged On
		self.wTree.get_widget('TimeLabel').set_label("Logon Time : "+userlist[3])
		self.wTree.get_widget('TimeBelowLabel').set_label("Logon Time : "+userlist[3])
		# Show/Hide Date and Time Labels
		if self.hidelogged == True:
			self.wTree.get_widget("DateLabel").hide()
			self.wTree.get_widget("DateBelowLabel").hide()
			self.wTree.get_widget("TimeLabel").hide()
			self.wTree.get_widget("TimeBelowLabel").hide()
		else:
			self.wTree.get_widget("DateLabel").show()
			self.wTree.get_widget("DateBelowLabel").show()
			self.wTree.get_widget("TimeLabel").show()
			self.wTree.get_widget("TimeBelowLabel").show()
		# Sort Out Photo Position By Hiding VBox
		if self.namebelow == True:
			self.wTree.get_widget("vbox1").hide()
			self.wTree.get_widget("vbox2").show()
		else:
			self.wTree.get_widget("vbox1").show()
			self.wTree.get_widget("vbox2").hide()
		# Show/Hide Username
		if self.hidename == True:
			self.wTree.get_widget("UserLabel").hide()
			self.wTree.get_widget("UserBelowLabel").hide()
		else:
			self.wTree.get_widget("UserLabel").show()
			self.wTree.get_widget("UserBelowLabel").show()


	def do_plugin( self ):
		self.RegenPlugin()
		self.SetupEvents()
		self.showuser()

#---------------------------------------------------------------------------------------#
# USPconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "uspuser.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "User"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/usp/plugins/user'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/usp/plugins/user', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_UserFontSpin_changed" : self.ufontspin,
				"on_UserHSpin_changed" : self.hspin,
				"on_UserWSpin_changed" : self.wspin,
				"on_UserBoldChkBtn_toggled" : self.uboldchk,
				"on_HideLoggedChkBtn_toggled" : self.hidelog,
				"on_HideNameChkBtn_toggled" : self.hidename,
				"on_FullNameChkBtn_toggled" : self.showfullname,
				"on_NameBelowChkBtn_toggled" : self.namebelow,
				"on_AnimPicEntry_changed" : self.useanimpic,
				"on_UserIconEntry_changed" : self.usericonchange,
				"on_StickyChkBtn_toggled" : self.usersticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("UserFontSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/user/settings_font_size',14))
		self.wTree.get_widget("UserHSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/user/height',200))
		self.wTree.get_widget("UserWSpin").set_value(SetGconf(self.client,'int','/apps/usp/plugins/user/width',180))
		self.wTree.get_widget("UserBoldChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/settings_font_bold',False))
		self.wTree.get_widget("NameBelowChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/name_below',False))
		self.wTree.get_widget("HideLoggedChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/hide_logged_on',False))
		self.wTree.get_widget("HideNameChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/hide_user_name',False))
		self.wTree.get_widget("FullNameChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/show_full_name',False))
		self.wTree.get_widget("AnimPicEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/user/animated_picture',''))
		self.wTree.get_widget("UserIconEntry").set_text(SetGconf(self.client,'string','/apps/usp/plugins/user/icon','login-photo'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','/apps/usp/plugins/user/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def ufontspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/user/settings_font_size',int(self.wTree.get_widget("UserFontSpin").get_value()))

	def hspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/user/height',int(self.wTree.get_widget("UserHSpin").get_value()))

	def wspin(self, widget, **kargs):
	        self.client.set_int('/apps/usp/plugins/user/width',int(self.wTree.get_widget("UserWSpin").get_value()))

	def uboldchk(self, widget, **kargs):
		if self.wTree.get_widget("UserBoldChkBtn").get_active() == True:
		        self.client.set_bool('/apps/usp/plugins/user/settings_font_bold',True)
		else:
		        self.client.set_bool('/apps/usp/plugins/user/settings_font_bold',False)

	def namebelow(self, widget, **kargs):
		if self.wTree.get_widget("NameBelowChkBtn").get_active() == True:
		        self.client.set_bool('/apps/usp/plugins/user/name_below',True)
		else:
		        self.client.set_bool('/apps/usp/plugins/user/name_below',False)

	def hidelog(self, widget, **kargs):
		if self.wTree.get_widget("HideLoggedChkBtn").get_active() == True:
		        self.client.set_bool('/apps/usp/plugins/user/hide_logged_on',True)
		else:
		        self.client.set_bool('/apps/usp/plugins/user/hide_logged_on',False)

	def hidename(self, widget, **kargs):
		if self.wTree.get_widget("HideNameChkBtn").get_active() == True:
		        self.client.set_bool('/apps/usp/plugins/user/hide_user_name',True)
		else:
		        self.client.set_bool('/apps/usp/plugins/user/hide_user_name',False)

	def showfullname(self, widget, **kargs):
		if self.wTree.get_widget("FullNameChkBtn").get_active() == True:
		        self.client.set_bool('/apps/usp/plugins/user/show_full_name',True)
		else:
		        self.client.set_bool('/apps/usp/plugins/user/show_full_name',False)

	def useanimpic(self, widget, **kargs):
		self.client.set_string('/apps/usp/plugins/user/animated_picture',self.wTree.get_widget("AnimPicEntry").get_text())

	def usericonchange(self, widget, **kargs): 
		self.icon=self.wTree.get_widget("UserIconEntry").get_text()
		self.client.set_string('/apps/usp/plugins/user/icon',self.wTree.get_widget('UserIconEntry').get_text())

	def usersticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/usp/plugins/user/sticky',True)
		else:
			self.client.set_bool('/apps/usp/plugins/user/sticky',False)
